#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import re
import os
import time
import logging
from . import config
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
import watchdog.events as events
from threading import Thread
from .cleepapi import CleepApi
from .file import File
from collections import deque

class ActionFileSync():
    def __init__(self, module=None):
        self.module = module

class ActionRestart():
    def __init__(self, frontend=False):
        self.frontend = frontend


class ActionsExecutor(Thread):
    """
    Actions executor
    Implements queue to execute sequentially actions
    """
    def __init__(self):
        Thread.__init__(self)
        Thread.daemon = True

        self.logger = logging.getLogger(self.__class__.__name__)
        self.running = True
        self.__queue = deque(maxlen=200)
        self.file = File()
        self.cleep = CleepApi()

    def stop(self):
        """
        Stop process
        """
        self.running = False

    def add_action(self, action):
        """
        Add an action to execute
        """
        self.__queue.appendleft(action)

    def __execute_action(self, action):
        """
        Execute specified action

        Args:
            action (dict): action to execute
        """
        if isinstance(action, ActionFileSync):
            if action.module is None:
                self.file.core_sync()
            else:
                self.file.module_sync(action.module)

        elif isinstance(action, ActionRestart):
            if action.frontend:
                self.cleep.restart_frontend()
            else:
                self.cleep.restart_backend()

    def run(self):
        """
        Main process: unqueue action and process it
        """
        while self.running:
            try:
                action = self.__queue.pop()
                self.__execute_action(action)

            except IndexError:
                #no action available
                time.sleep(0.25)

            except:
                #error occured during action execution
                self.logger.exception('Error during action execution:')


class CleepHandler(FileSystemEventHandler):
    """
    Watchdog handler for Cleep
    """
    REJECTED_FILENAMES = [
        u'4913', #vim furtive temp file to check user permissions
        u'.gitignore'
    ]
    REJECTED_EXTENSIONS = [
        u'.swp', #vim
        u'.swpx', #vim
        u'.swx', #vim
        u'.tmp', #generic?
        u'.offset', #pygtail
        u'.pyc', #python compiled
        u'.log' #log file
    ]
    REJECTED_PREFIXES = [
        u'.',
        u'~'
    ]
    REJECTED_SUFFIXES = [
        u'~'
    ]
    REJECTED_DIRS = [
        u'.git',
        u'.vscode',
        u'.editor',
        u'backend/__pycache__',
    ]
    DEBOUNCE = 0.25 #seconds

    def __init__(self, actions_executor):
        self.__last_times = {
            'backend': 0,
            'frontend': 0
        }
        self.logger = logging.getLogger(self.__class__.__name__)
        self.actions_executor = actions_executor

    def __is_event_dropped(self, event):
        """
        Analyse event and return True if event must be dropped
        Return:
            bool: True if event must be dropped
        """
        #drop dir modified event or created file (that always comes with modified file)
        if type(event) in (events.DirModifiedEvent, events.FileCreatedEvent):
            self.logger.debug('Filtered: DirModifiedEvent or FileCreatedEvent')
            return True

        #filter invalid event
        if not event:
            self.logger.debug('Filtered: invalid event')
            return True

        #filter event on current script
        if event.src_path == u'.%s' % __file__:
            self.logger.debug('Filtered: event on current script')
            return True

        #filter root event
        if event.src_path == u'.':
            self.logger.debug('Filtered: root event')
            return True

        #filter invalid extension
        src_ext = os.path.splitext(event.src_path)[1]
        if src_ext in self.REJECTED_EXTENSIONS:
            self.logger.debug('Filtered: invalid extension')
            return True

        #filter by prefix
        for prefix in self.REJECTED_PREFIXES:
            if event.src_path.startswith(prefix):
                self.logger.debug('Filtered: source prefix')
                return True
            if getattr(event, u'dest_path', None) and event.dest_path.startswith(prefix):
                self.logger.debug('Filtered: destination prefix')
                return True

        #filter by suffix
        for suffix in self.REJECTED_SUFFIXES:
            if event.src_path.endswith(suffix):
                self.logger.debug('Filtered: source suffix')
                return True
            if getattr(event, u'dest_path', None) and event.dest_path.endswith(prefix):
                self.logger.debug('Filtered: destination suffix')
                return True

        #filter by filename
        for filename in self.REJECTED_FILENAMES:
            if event.src_path.endswith(filename):
                self.logger.debug('Filtered: filename')
                return True

        #filter by dir
        parts = event.src_path.split(os.path.sep)
        for folder in self.REJECTED_DIRS:
            if folder in parts:
                self.logger.debug('Filtered: directory')
                return True

        return False

    def __debounce(self, now, last_time):
        if now<(last_time+self.DEBOUNCE):
            return True
        return False

    def __change_on_module(self, event):
        """
        Change on module

        Args:
            event: watchdog event

        Returns:
            module name (string) or None if nothing found
        """
        pattern = '^%s/(.*?)/(?:frontend|backend|scripts)/.*$' % (config.MODULES_SRC)
        #self.logger.debug('Module pattern: %s' % pattern)

        if hasattr(event, 'src_path'):
            res = re.findall(pattern, event.src_path)
            if res and len(res)>0:
                return res[0]

        if hasattr(event, 'dest_path'):
            res = re.findall(pattern, event.dest_path)
            if res and len(res)>0:
                return res[0]
        
        return None

    def __change_on_core(self, event):
        """
        Change on core

        Args:
            event: watchdog event

        Returns:
            True if change occurs in core, False otherwise
        """
        pattern_core = '^(%s/(?!tests|modules)(.*\.py)|%s/.*)$' % (config.CORE_SRC, config.HTML_SRC)
        pattern_bin = '^%s/cleep' % (config.BIN_SRC)
        #self.logger.debug('Core pattern: %s' % pattern_core)
        #self.logger.debug('Bin pattern: %s' % pattern_bin)

        if hasattr(event, 'src_path'):
            if re.search(pattern_core, event.src_path):
                return True
            if re.search(pattern_bin, event.src_path):
                return True

        if hasattr(event, 'dest_path'):
            if re.search(pattern_core, event.dest_path):
                return True

        return False

    def __change_on_frontend(self, event):
        """
        Detect change on frontend

        Args:
            event: watchdog event

        Returns:
            True if change on frontend, False otherwise
        """
        pattern = '^(%s/|%s/(?:.*?)/frontend/).*$' % (config.HTML_SRC, config.MODULES_SRC)
        #self.logger.debug('Frontend pattern: %s' % pattern)

        if hasattr(event, 'src_path'):
            res = re.search(pattern, event.src_path)
            if res:
                return True

        if hasattr(event, 'dest_path'):
            res = re.search(pattern, event.dest_path)
            if res:
                return True

        return False

    def on_any_event(self, event):
        try:
            now = time.time()
            self.logger.debug('Event: %s' % event)
            if not self.__is_event_dropped(event):
                self.__last_action_time = now

                #is event on backend or frontend ?
                frontend = self.__change_on_frontend(event)

                #debounce ?
                if self.__debounce(now, self.__last_times[('frontend' if frontend else 'backend')]):
                    self.logger.debug('  Dropped by debounce')
                    return 

                #is event on module ?
                change_on_core = False
                change_on_module = self.__change_on_module(event)
                if change_on_module is None:
                    change_on_core = self.__change_on_core(event)

                #push action and update last action time
                if change_on_module:
                    self.logger.debug('  Change on module "%s"' % change_on_module)
                    self.actions_executor.add_action(ActionFileSync(change_on_module))
                elif change_on_core:
                    self.logger.debug('  Change on core')
                    self.actions_executor.add_action(ActionFileSync())
                else:
                    self.logger.debug('  Dropped due to useless change')
                    return
                if frontend:
                    self.logger.debug('  Change on frontend')
                    self.actions_executor.add_action(ActionRestart(True))
                    self.__last_times['frontend'] = now
                else:
                    self.logger.debug('  Change on backend')
                    self.actions_executor.add_action(ActionRestart())
                    self.__last_times['backend'] = now

        except:
            self.logger.exception('Error occured during watcher event processing:')


class CleepWatchdog():
    """
    Cleep core and module watchdog
    """
    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.actions_executor = ActionsExecutor()

    def watch(self):
        """
        Start watchdog that monitors filesystem changes on Cleep sources
        """
        path = '%s/' % config.REPO_DIR
        self.actions_executor.start()

        if not os.path.exists(path):
            self.logger.error('Please get Cleep sources using "cleep-cli getcore" command')
            return False

        event_handler = CleepHandler(self.actions_executor)
        observer = Observer()
        observer.schedule(event_handler, path, recursive=True)
        self.logger.info('Cleep watchdog is running on "%s" (CTRL-C to stop)' % path)
        observer.start()
        try:
            while True:
                time.sleep(0.25)
        except KeyboardInterrupt:
            observer.stop()
            self.actions_executor.stop()
        observer.join()

        return True




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
from .console import Console
import logging
from . import config

class Git():
    """
    Git commands
    """

    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)

    def pull_core(self):
        """
        Pull core content
        """
        self.logger.info('Pulling core repository...')
        c = Console()
        cmd = 'cd "%s"; git pull -q' % config.REPO_DIR
        self.logger.debug('cmd: %s' % cmd)
        resp = c.command(cmd, 60)
        self.logger.debug('Pull resp: %s' % resp)
        if resp['error'] or resp['killed']:
            self.logger.error('Error occured while pulling repository: %s' % ('killed' if resp['killed'] else resp['stderr']))
            return False

        if not os.path.exists(os.path.join(config.REPO_DIR, 'modules')):
            os.mkdir(os.path.join(config.REPO_DIR, 'modules'))

        self.logger.info('Done')
        return True

    def clone_core(self):
        """
        Clone core content from official repository
        """
        # clone repo
        self.logger.info('Cloning core repository...')
        c = Console()
        url = config.REPO_URL
        cmd = 'git clone -q "%s" "%s"' % (url, config.REPO_DIR)
        self.logger.debug('cmd: %s' % cmd)
        resp = c.command(cmd, 60)
        self.logger.debug('Clone resp: %s' % resp)
        if resp['error'] or resp['killed']:
            self.logger.error('Error occured while cloning repository: %s' % ('killed' if resp['killed'] else resp['stderr']))
            return False
        
        if not os.path.exists(os.path.join(config.REPO_DIR, 'modules')):
            os.mkdir(os.path.join(config.REPO_DIR, 'modules'))

        self.logger.info('Done')
        return True

    def pull_mod(self, module):
        """
        Pull module content

        Args:
            module (string): module name
        """
        self.logger.info('Pulling "%s" module repository...' % module)
        c = Console()
        module_path = os.path.join(config.MODULES_SRC, module)
        cmd = 'cd "%s"; git pull -q' % (module_path)
        self.logger.debug('cmd: %s' % cmd)
        resp = c.command(cmd, 60)
        self.logger.debug('Pull resp: %s' % resp)
        if resp['error'] or resp['killed']:
            self.logger.error('Error occured while pulling "%s" mod repository: %s' % (module, 'killed' if resp['killed'] else resp['stderr']))
            return False

        self.logger.info('Done')
        return True

    def clone_mod(self, module):
        """
        Clone core content from official repository

        Args:
            module (string): module name
        """
        self.logger.info('Cloning "%s" module repository...' % module)
        c = Console()
        url = config.MODULES_REPO_URL[module]
        module_path = os.path.join(config.MODULES_SRC, module)
        cmd = 'git clone -q "%s" "%s"' % (url, module_path)
        self.logger.debug('cmd: %s' % cmd)
        resp = c.command(cmd, 60)
        self.logger.debug('Clone resp: %s' % resp)
        if resp['error'] or resp['killed']:
            self.logger.error('Error occured while cloning "%s" mod repository: %s' % (module, 'killed' if resp['killed'] else resp['stderr']))
            return False
        
        self.logger.info('Done')
        return True




try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
from cleepcli.version import VERSION

requirements = []
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name = 'cleepcli',
    version = VERSION,
    description = 'Cleep-cli helps developers to build great Cleep applications from command line.',
    author = 'Tanguy Bonneau',
    author_email = 'tanguy.bonneau@gmail.com',
    maintainer = 'Tanguy Bonneau',
    maintainer_email = 'tanguy.bonneau@gmail.com',
    url = 'http://www.github.com/tangb/cleep-cli/',
    packages = ['cleepcli'],
    include_package_data = True,
    install_requires = requirements,
    scripts = ['bin/cleep-cli', 'bin/ccli']
)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
import logging
from . import config
from .console import Console
import requests
import json

class CleepApi():
    """
    Cleep api helper
    """

    COMMAND_URL = 'http://localhost/command'

    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)

    def restart_backend(self):
        """
        Send command to restart backend
        """
        self.logger.info('Restarting backend')

        #/!\ old way to restart cleep is not sure if core is broken
        #data = {'to':'system', 'command':'restart', 'delay':0.0}
        #self.__post(self.COMMAND_URL, data)

        cmd = '/bin/systemctl restart cleep'
        c = Console()
        resp = c.command(cmd)
        self.logger.debug('Systemctl resp: %s' % resp)
        if resp['error'] or resp['killed']:
            self.logger.error('Error restarting cleep backend')
            return False

        return True

    def restart_frontend(self):
        """
        Send command to restart frontend
        """
        self.logger.info('Restarting frontend')
        data = {'to':'developer', 'command':'restart_frontend'}
        self.__post(self.COMMAND_URL, data)

    def __post(self, url, data):
        """
        Post data to specified url

        Args:
            url (string): request url
            data (dict): request data
        """
        try:
            resp = requests.post(url, json=data)
            resp_data = resp.json()
            self.logger.debug('Response[%s]: %s', resp.status_code, resp_data)
            return (resp.status_code, resp_data)
        except Exception as e:
            if self.logger.getEffectiveLevel()==logging.DEBUG:
                self.logger.exception('Error occured while requesting "%s"' % url)
            else:
                self.logger.error('Error occured while requesting "%s": %s' % (url, str(e)))




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import zipfile
import os
import glob
import re
import requests
import time
from . import config
from .console import Console
from .check import Check
import subprocess

class Ci():
    """
    Continuous Integration helpers
    """

    CLEEP_COMMAND_URL = 'http://127.0.0.1/command'
    CLEEP_HEALTH_URL = 'http://127.0.0.1/health'
    TESTS_REQUIREMENTS_TXT = 'tests/requirements.txt'
    EXTRACT_DIR = '/tmp/extract'
    APP_FILENAME_PATTERN = 'APP_FILENAME=[\'"](.*)[\'"]'

    def __init__(self):
        """
        Constructor
        """
        self.logger = logging.getLogger(self.__class__.__name__)

    def mod_check_package(self, package_path):
        """
        Check specified package content

        Args:
            package_path (string): package path
            
        Returns:
            dict: package informations::

            {
                module_name (str): module name
                module_version (str): module version
                has_tests_requirements (bool): True if tests/requirements.txt exists
            }

        Raises:
            Exception if something bad detected
        """
        self.logger.info('Checking application package "%s"' % package_path)
        (_, module_name, module_version) = os.path.basename(package_path).split('_')
        module_version = module_version.replace('.zip', '')[1:]

        # check raw file
        if not module_version:
            raise Exception('Invalid package filename')
        if not re.match('\d+\.\d+\.\d+', module_version):
            raise Exception('Invalid package filename')
        console = Console()
        resp = console.command('file --keep-going --mime-type "%s"' % package_path)
        if resp['returncode'] != 0:
            raise Exception('Unable to check file validity')
        filetype = resp['stdout'][0].split(': ')[1].strip()
        self.logger.debug('Filetype=%s' % filetype)
        if filetype != 'application/zip\\012- application/octet-stream':
            raise Exception('Invalid application package file')

        # check package structure
        has_tests_requirements = False
        checks = {
            'dir_backend': False,
            'dir_frontend': False,
            'dir_tests': False,
            'file_module_json': False,
            'file_desc_json': False,
            'file_module_py': False,
            'file_backend_init_py': False,
            'file_tests_init_py': False,
        }
        with zipfile.ZipFile(package_path, 'r') as zp:
            for zfile in zp.infolist():
                if zfile.filename.startswith('backend/modules/%s' % module_name):
                    checks['dir_backend'] = True
                if zfile.filename.startswith('frontend/js/modules/%s' % module_name):
                    checks['dir_frontend'] = True
                if zfile.filename.startswith('tests'):
                    checks['dir_tests'] = True
                if zfile.filename == 'backend/modules/%s/%s.py' % (module_name, module_name):
                    checks['file_module_py'] = True
                if zfile.filename == 'frontend/js/modules/%s/desc.json' % module_name:
                    checks['file_desc_json'] = True
                if zfile.filename == 'module.json':
                    checks['file_module_json'] = True
                if zfile.filename == self.TESTS_REQUIREMENTS_TXT:
                    has_tests_requirements = True
                if zfile.filename == 'backend/modules/%s/__init__.py' % module_name:
                    checks['file_backend_init_py'] = True
                if zfile.filename == 'tests/__init__.py':
                    checks['file_tests_init_py'] = True

            # special case for custom main module filename
            if checks['file_backend_init_py'] and not checks['file_module_py']:
                content = zp.read('backend/modules/%s/__init__.py' % module_name).decode('utf8').strip()
                matches = re.findall(self.APP_FILENAME_PATTERN, content)
                if len(matches) == 1 and 'backend/modules/%s/%s.py' % (module_name, matches[0]) in zp.namelist():
                    checks['file_module_py'] = True

        self.logger.debug('Checks results: %s' % checks)
        if not all(checks.values()):
            raise Exception('Invalid package structure. Make sure to build it with developer application')

        return {
            'module_name': module_name,
            'module_version': module_version,
            'has_test_requirements': has_tests_requirements,
        }

    def mod_install_source(self, package_path, package_infos, no_compatibility_check=False):
        """
        Install module package (zip archive) sources

        Args:
            package_path (string): package path
            package_infos (dict): infos returned by mod_check_package function
            no_compatitiblity_check (bool): do not check module compatibility (but only deps compat)

        Raises:
            Exception if error occured
        """
        module_name, module_version, has_tests_requirements = package_infos.values()
        self.logger.info('Installing application %s v%s...' % (module_name, module_version))

        try:
            # start cleep (non blocking)
            self.logger.info('  Starting Cleep...')
            cleep_proc = subprocess.Popen(['cleep', '--noro'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            time.sleep(15)

            # make sure to have latest modules.json version
            self.logger.info('  Updating applications list in Cleep')
            resp = requests.post(self.CLEEP_COMMAND_URL, json={
                'command': 'check_modules_updates',
                'to': 'update',
            })
            resp.raise_for_status()
            resp_json = resp.json()
            if resp_json['error']:
                raise Exception('Check_modules_updates command failed: %s' % resp_json)

            # update "update" module to enjoy bug fixes
            self.logger.info('  Updating "update" application...')
            resp = requests.post(self.CLEEP_COMMAND_URL, json={
                'command': 'update_module',
                'to': 'update',
                'params': {
                    'module_name': 'update',
                }
            })
            resp.raise_for_status()
            resp_json = resp.json()
            self.logger.debug('Update "update" resp: %s' % resp_json)
            if resp_json['error']:
                raise Exception('Updating app "update" failed: %s' % resp_json)
            if resp_json['data']:
                self.__wait_for_cleep_process('update')
                self.logger.info('  Restarting cleep...')
                cleep_proc.kill()
                cleep_proc = subprocess.Popen(['cleep', '--noro'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                time.sleep(15)

            # install module in cleep (it will also install deps)
            self.logger.info('  Installing "%s" application in Cleep' % module_name)
            resp = requests.post(self.CLEEP_COMMAND_URL, json={
                'command': 'install_module',
                'to': 'update',
                'params': {
                    'module_name': module_name,
                    'package': package_path,
                    'no_compatibility_check': no_compatibility_check,
                }
            })
            resp.raise_for_status()
            resp_json = resp.json()
            if resp_json['error']:
                raise Exception('Installing "%s" app failed: %s' % (module_name, resp_json))
            self.__wait_for_cleep_process(module_name)

            # restart cleep
            self.logger.info('  Restarting cleep...')
            cleep_proc.kill()
            cleep_proc = subprocess.Popen(['cleep', '--noro'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            time.sleep(15)

            # check module is installed and running
            self.logger.info('  Checking application is installed...')
            resp = requests.get(self.CLEEP_HEALTH_URL)
            self.logger.info('Health response: %s', resp.json())
            resp.raise_for_status()
            self.logger.info('Application and its dependencies successfully installed')

            # install requirements.txt for tests
            if has_tests_requirements:
                self.logger.info('Installing tests python dependencies...')
                console = Console()
                requirements_txt_path = os.path.join(config.MODULES_SRC, module_name, self.TESTS_REQUIREMENTS_TXT)
                self.logger.debug('Tests requirements.txt path "%s"', requirements_txt_path)
                resp = console.command('python3 -m pip install --trusted-host pypi.org -r "%s"' % requirements_txt_path, 900)
                self.logger.debug('Resp: %s' % resp)
                if resp['returncode'] != 0:
                    self.logger.error('Error installing tests requirements.txt: %s' , resp)
                    raise Exception('Error installing tests requirements.txt (killed=%s)' % resp['killed'])

        finally:
            if cleep_proc:
                cleep_proc.kill()

    def __wait_for_cleep_process(self, module_name):
        """
        Wait for end of current Cleep process (install, update...)
        """
        while True:
            time.sleep(1.0)
            resp = requests.post(self.CLEEP_COMMAND_URL, json={
                'command': 'get_modules_updates',
                'to': 'update'
            })
            resp.raise_for_status()
            resp_json = resp.json()
            if resp_json['error']:
                raise Exception('Get_modules_updates command failed')
            module_updates = resp_json['data'].get(module_name)
            self.logger.debug('Updates: %s' % module_updates)
            if not module_updates:
                raise Exception('No "%s" application info in updates data' % module_name)
            if module_updates['processing'] == False:
                if module_updates['update']['failed']:
                    raise Exception('Application "%s" installation failed' % module_name)
                break

    def mod_extract_sources(self, package_path, package_infos):
        """
        Extract module package (zip archive) in dev directory

        Args:
            package_path (string): package path
            package_infos (dict): infos returned by mod_check_package function

        Raises:
            Exception if error occured
        """
        module_name, module_version, _ = package_infos.values()

        # unzip content
        self.logger.info('Extracting archive "%s" to "%s"' % (package_path, self.EXTRACT_DIR))
        with zipfile.ZipFile(package_path, 'r') as package:
            package.extractall(self.EXTRACT_DIR)

        # install sources
        os.makedirs(os.path.join(config.MODULES_SRC, module_name), exist_ok=True)
        for filepath in glob.glob(self.EXTRACT_DIR + '/**/*.*', recursive=True):
            if filepath.startswith(os.path.join(self.EXTRACT_DIR, 'frontend')):
                dest = filepath.replace(os.path.join(self.EXTRACT_DIR, 'frontend/js/modules/%s' % module_name), os.path.join(config.MODULES_SRC, module_name, 'frontend'))
                self.logger.debug(' -> frontend: %s' % dest)
            elif filepath.startswith(os.path.join(self.EXTRACT_DIR, 'backend')):
                dest = filepath.replace(os.path.join(self.EXTRACT_DIR, 'backend/modules/%s' % module_name), os.path.join(config.MODULES_SRC, module_name, 'backend'))
                self.logger.debug(' -> backend: %s' % dest)
            elif filepath.startswith(os.path.join(self.EXTRACT_DIR, 'tests')):
                dest = filepath.replace(os.path.join(self.EXTRACT_DIR, 'tests'), os.path.join(config.MODULES_SRC, module_name, 'tests'))
                self.logger.debug(' -> tests: %s' % dest)
            elif filepath.startswith(os.path.join(self.EXTRACT_DIR, 'scripts')):
                dest = filepath.replace(os.path.join(self.EXTRACT_DIR, 'scripts'), os.path.join(config.MODULES_SRC, module_name, 'scripts'))
                self.logger.debug(' -> scripts: %s' % dest)
            else:
                dest = filepath.replace(self.EXTRACT_DIR, os.path.join(config.MODULES_SRC, module_name))
                self.logger.debug(' -> other: %s' % dest)
            os.makedirs(os.path.dirname(dest), exist_ok=True)
            os.rename(filepath, dest)

    def mod_check(self, module_name):
        """
        Perform some checkings (see check.py file) for continuous integration

        Args:
            module_name (string): module name

        Raises:
            Exception if error occured
        """
        check = Check()

        check.check_backend(module_name)
        check.check_frontend(module_name)
        check.check_scripts(module_name)
        check.check_tests(module_name)

